-- Setup Variables
local bcsQuickFrame
local bcsDetailedFrame
local bossCheatSheetsPlug = "Visit BossCheatSheets.com to submit/vote on strategies!"
local selectedChatID = 1
local expansionId = "6"
local zoneId
local bossId
local specId
local expansions = Expansions
local zones = Zones
local bosses = Bosses
local bossids = BossIDs
local specializations = Specializations
local strategy_summary = StrategySummary
local strategy_summary_clean = StrategySummaryClean
local strategy_detailed = StrategyDetailed
local strategy_specialized_summary = StrategySpecializedSummary
local strategy_specialized_detailed = StrategySpecializedDetailed
local strategy_specialized_summary_clean = StrategySpecializedSummaryClean
local chatTextCleaned = ""
local eventsFrame = CreateFrame("FRAME", "eventsFrame");
local targetedBossId


local postOptions = {
  "Send strategy to raid",
  "Send strategy to party",
  "Send strategy to instance (LFG/LFR)",
  "Whisper strategy to: "
  --"'Say' strategy to everyone"
}

local function eventHandler(self, event, ...)
  local guid, name = UnitGUID("target"), UnitName("target")
  if (guid == nil) then
    targetedBossId = nil
  else
    local type, zero, server_id, instance_id, zone_uid, npc_id, spawn_uid = strsplit("-",guid)
    --print(name, "is a", type, "with an id of", npc_id)
    if (UnitAffectingCombat("player") or UnitIsDead("target")) then
    	-- Player in combat or Boss is dead. Dont show UI. Hide UI?
    else
	    targetedBossId = bossids[npc_id]
	    if not (targetedBossId == nil) then
	      if not (bcsQuickFrame == nil) then
  		    ClearHTMLFrames()
	      end
	      ShowTargetedBoss()
	    end
	end
  end
end

function BossCheatSheets_Onload()
  eventsFrame:RegisterEvent("PLAYER_TARGET_CHANGED");
  eventsFrame:SetScript("OnEvent", eventHandler);
end

function BossCheatSheets_Onshow()
  createQuickHtmlFrame()
  createDetailedHtmlFrame()
  ClearHTMLFrames()
end

function createQuickHtmlFrame()
  bcsQuickFrame  = CreateFrame("SimpleHTML", "BCSQuickFrame", BossCheatSheets_FramesDialog_TabPage1_ScrollFrame_ScrollChildFrame)
  bcsQuickFrame.width  = BossCheatSheets_FramesDialog_TabPage1:GetWidth() - 30
  bcsQuickFrame.height = BossCheatSheets_FramesDialog_TabPage1:GetHeight()
  bcsQuickFrame:SetSize(bcsQuickFrame.width, bcsQuickFrame.height)
  bcsQuickFrame:SetFrameStrata("HIGH")
  bcsQuickFrame:SetPoint("LEFT", BossCheatSheets_FramesDialog_TabPage1_ScrollFrame_ScrollChildFrame, "LEFT", 15, -15)
  bcsQuickFrame:SetBackdropColor(0, 0, 0, 1)
  bcsQuickFrame:SetFont('Fonts\\FRIZQT__.TTF', 11)
end

function createDetailedHtmlFrame()
  bcsDetailedFrame  = CreateFrame("SimpleHTML", "BCSDetailedFrame", BossCheatSheets_FramesDialog_TabPage2_ScrollFrame_ScrollChildFrame)
  bcsDetailedFrame.width  = BossCheatSheets_FramesDialog_TabPage1:GetWidth() - 30
  bcsDetailedFrame.height = BossCheatSheets_FramesDialog_TabPage1:GetHeight()
  bcsDetailedFrame:SetSize(bcsDetailedFrame.width, bcsDetailedFrame.height)
  bcsDetailedFrame:SetFrameStrata("HIGH")
  bcsDetailedFrame:SetPoint("LEFT", BossCheatSheets_FramesDialog_TabPage2_ScrollFrame_ScrollChildFrame, "LEFT", 15, -15)
  bcsDetailedFrame:SetBackdropColor(0, 0, 0, 1)
  bcsDetailedFrame:SetFont('Fonts\\FRIZQT__.TTF', 11)
end

-- EXPANSION DROPDOWN
function ExpansionDropDown_Menu(frame, level, menuList)
   local info = UIDropDownMenu_CreateInfo()
   for k,v in pairs(expansions) do
      info = UIDropDownMenu_CreateInfo()
      info.text = v
      info.value = k
      info.func = ExpansionDropDownOnSelect
      UIDropDownMenu_AddButton(info, level)
   end
end

function ExpansionDropDownOnSelect(self, arg1, arg2, checked)
  UIDropDownMenu_SetSelectedID(ExpansionDropDownMenuButton, self:GetID())

  expansionId = self.value

  -- load in zones into dropdown
  ZoneDropDown_Menu()
  UIDropDownMenu_SetText(ZoneDropDownMenuButton, "Choose Zone...")
  BossDropDown_Menu()
  UIDropDownMenu_SetText(BossDropDownMenuButton, "Choose Boss...")
  ClearHTMLFrames()
end

-- ZONE DROPDOWN
function ZoneDropDown_Menu(frame, level, menuList)
  for k,v in pairs(zones) do
    if (string.sub(k, 1, 1) == expansionId) then
      if not zoneId then
        zoneId = k
      end
      local info = UIDropDownMenu_CreateInfo()
      info.text = v
      info.value = k
      info.func = ZoneDropDownOnSelect
      UIDropDownMenu_AddButton(info, 1)
    end
  end
end

function ZoneDropDownOnSelect(self, arg1, arg2, checked)
  UIDropDownMenu_SetSelectedID(ZoneDropDownMenuButton, self:GetID())

  zoneId = self.value

  -- load boss list
  BossDropDown_Menu()
  UIDropDownMenu_SetText(BossDropDownMenuButton, "Choose Boss...")
  ClearHTMLFrames()
end

-- BOSS DROPDOWN
function BossDropDown_Menu(frame, level, menuList)
  for k,v in pairs(bosses) do
    if (string.find(k, zoneId .. "_", 1, true) == 1) then
      if not zoneId then
        bossId = k
      end
      local info = UIDropDownMenu_CreateInfo()
      info.text = v
      info.value = k
      info.func = BossDropDownOnSelect
      UIDropDownMenu_AddButton(info, level)
    end
  end
end

function BossDropDownOnSelect(self, arg1, arg2, checked)
  ClearHTMLFrames()

  UIDropDownMenu_SetSelectedID(BossDropDownMenuButton, self:GetID())

  bossId = self.value

  SetStrategyText()
end

-- SPEC DROPDOWN
function SpecDropDown_Menu(frame, level, menuList)
  local info = UIDropDownMenu_CreateInfo()
  info.text = "General Strategy"
  info.value = "0"
  info.func = SpecDropDownOnSelect
  UIDropDownMenu_AddButton(info, level)
  for k,v in pairs(specializations) do
      info = UIDropDownMenu_CreateInfo()
      info.text = v
      info.value = k
      info.func = SpecDropDownOnSelect
      UIDropDownMenu_AddButton(info, level)
  end
end

function SpecDropDownOnSelect(self, arg1, arg2, checked)
	ClearHTMLFrames()

	UIDropDownMenu_SetSelectedID(SpecDropDownMenuButton, self:GetID())

	specId = self.value

	SetStrategyText()
end

-- SPEC DROPDOWN
function PostDropDown_Menu(frame, level, menuList)
   local info = UIDropDownMenu_CreateInfo()
   for k,v in pairs(postOptions) do
      info = UIDropDownMenu_CreateInfo()
      info.text = v
      info.value = k
      info.func = PostDropDownOnSelect
      UIDropDownMenu_AddButton(info, level)
   end
end

function PostDropDownOnSelect(self, arg1, arg2, checked)
	
	UIDropDownMenu_SetSelectedID(PostDropDownMenuButton, self.value)
	selectedChatID = self.value

	if (self.value == 1) then
		PostActionButton:SetText("Send")
		WhisperToEditBox:Disable()		
		WhisperToEditBox:Hide()
		WhisperToEditBox:SetText("")
	elseif (self.value == 2) then
		PostActionButton:SetText("Send")
		WhisperToEditBox:Disable()    
		WhisperToEditBox:Hide()
		WhisperToEditBox:SetText("")
	elseif (self.value == 3) then
		PostActionButton:SetText("Send")
		WhisperToEditBox:Disable()    
		WhisperToEditBox:Hide()
		WhisperToEditBox:SetText("")
	elseif (self.value == 4) then
		PostActionButton:SetText("Whisper")
		WhisperToEditBox:Show()
		WhisperToEditBox:Enable()
		WhisperToEditBox:SetText("<enter name>")
		WhisperToEditBox:SetFocus()
		WhisperToEditBox:HighlightText()
	elseif (self.value == 5) then
		PostActionButton:SetText("Say")
		WhisperToEditBox:Disable()		
		WhisperToEditBox:Hide()
		WhisperToEditBox:SetText("")
	end
end

function PostActionButtonClick()
	strategy = chatTextCleaned
	lastWordIndex = 1
	nextWordIndex = 255

	postTo = "WHISPER"
	whisperName = nil

	if (selectedChatID == 1) then
		postTo = "RAID"
  elseif (selectedChatID == 2) then
    postTo = "PARTY"
  elseif (selectedChatID == 3) then
    postTo = "INSTANCE_CHAT"
  elseif (selectedChatID == 4) then
    postTo = "WHISPER"
		whisperName = WhisperToEditBox:GetText()
	elseif(selectedChatID == 5) then
		postTo = "SAY"
	end

	while (true) -- Break at word boundary before 255 chars. This is for WoW chat limits
	do
		if (nextWordIndex > string.len(strategy)) then
			nextWordIndex = string.len(strategy)
		else
			savedWordIndex = nextWordIndex --just in case no word boundaries are met
			--TODO: Add other word boundaries like period, comma, dash, quotes, etc... Maybe use a regex instead
			while (string.byte(strategy, nextWordIndex) ~= 32) --32 is a space character. Reverse until one is found (a word boundary)
			do
				if (nextWordIndex <= lastWordIndex) then
					nextWordIndex = savedWordIndex -- There were no word boundaries left. Just take what's there
					break;
				end
				nextWordIndex = nextWordIndex - 1
			end
		end
		stratChunk = string.sub(strategy, lastWordIndex, nextWordIndex)

		SendChatMessage(stratChunk, postTo, "Common", whisperName)
		lastWordIndex = nextWordIndex
		nextWordIndex = nextWordIndex + 255
		if (lastWordIndex >= string.len(strategy)) then
			break
		end
	end	
	
	SendChatMessage(bossCheatSheetsPlug, postTo, "Common", whisperName)	
end

function SetStrategyText()
  if (SpecDropDownMenuButton:GetID() == 0) then
    if not (strategy_summary[bossId] == nil) then
      bcsQuickFrame:SetText("<html><body>" .. strategy_summary[bossId] .. "</body></html>")
      chatTextCleaned = strategy_summary_clean[bossId]
    else
      bcsQuickFrame:SetText("<html><body><p>No quick strategy found. Write one at bosscheatsheets.com!</p></body></html>")
      chatTextCleaned = ""
    end
    if not (strategy_detailed[bossId] == nil) then
      bcsDetailedFrame:SetText("<html><body>" .. strategy_detailed[bossId] .. "</body></html>")
    else
      bcsDetailedFrame:SetText("<html><body><p>No detailed strategy found. Write one at bosscheatsheets.com!</p></body></html>")
    end
  else
    if strategy_specialized_summary[bossId .. "_" .. specId] then
      bcsQuickFrame:SetText("<html><body>" .. strategy_summary[bossId .. "_" .. specId] .. "</body></html>")
      chatTextCleaned = strategy_specialized_summary_clean[bossId]
    else
      bcsQuickFrame:SetText("<html><body><p>No specific strategy was found for " .. SpecDropDownMenuButton:GetSelectedText() .. ". <br />Try a more general strategy. Or write one at bosscheatsheets.com!</p></body></html>")
      chatTextCleaned = ""
    end
    if strategy_specialized_detailed[bossId .. "_" .. specId] then
      bcsDetailedFrame:SetText("<html><body>" .. strategy_detailed[bossId .. "_" .. specId] .. "</body></html>")
    else
      bcsDetailedFrame:SetText("<html><body><p>No specific strategy was found for " .. SpecDropDownMenuButton:GetSelectedText() .. ". <br />Try a more general strategy. Or write one at bosscheatsheets.com!</p></body></html>")
    end
  end
end

function ClearHTMLFrames()
  bcsQuickFrame:SetText("<html><body><p></p></body></html>")
  bcsDetailedFrame:SetText("<html><body><p></p></body></html>")	
end

function ShowTargetedBoss()
  if not (targetedBossId == nil) then
    local expansionPart, zonePart, bossPart = strsplit("_", targetedBossId)
    expansionId = expansionPart
    zoneId = expansionPart .. "_" .. zonePart
    bossId = targetedBossId

    ShowUIPanel(BossCheatSheets)    

    ExpansionDropDown_Menu()
    UIDropDownMenu_SetSelectedValue(ExpansionDropDownMenuButton, expansionId)
    
    ZoneDropDown_Menu()
    UIDropDownMenu_SetSelectedValue(ZoneDropDownMenuButton, zoneId)

    BossDropDown_Menu()
    UIDropDownMenu_SetSelectedValue(BossDropDownMenuButton, bossId)

    SetStrategyText()
  end
end

-- Open dialog box when /bcs is entered
--TODO: Remove Chat text
SLASH_BOSSCHEATSHEETS1 = "/bcs"
SlashCmdList["BOSSCHEATSHEETS"] = function(msg, editbox)
  ShowUIPanel(BossCheatSheets)
end